//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Security;
using System.Security.Permissions;
using System.Text;

namespace CLRUnleashed
{
	/// <summary>
	/// Summary description for CustomPermission.
	/// </summary>
	[Serializable()]
	public class CustomPermission: CodeAccessPermission , IUnrestrictedPermission
	{
		private bool unrestricted;

		public CustomPermission(PermissionState state)
		{
			if(state == PermissionState.Unrestricted)
			{
				unrestricted = true;
			}
			else
			{
				unrestricted = false;
			}
		}
      
		public bool IsUnrestricted()
		{
			return unrestricted;
		}

		public override IPermission Copy()
		{
			//Create a new instance of CustomPermission with the current
			//value of unrestricted.
			CustomPermission copy = new CustomPermission(PermissionState.None);

			if(this.IsUnrestricted())
			{
				copy.unrestricted = true;
			}
			else
			{
				copy.unrestricted = false;
			}
			//Return the copy.
			return copy;
		}

		public override IPermission Intersect(IPermission target)
		{
			//If nothing was passed, return null.
			if(null == target)
			{
				return null;
			}
			try
			{
				//Create a new instance of CustomPermission from the passed object.
				CustomPermission PassedPermission = (CustomPermission)target;

				//If one class has an unrestricted value of false, then the
				//intersection will have an unrestricted value of false.
				//Return the passed class with the unrestricted value of false.
				if(!PassedPermission.unrestricted)
				{
					return target;
				}
				//Return a copy of the current class if the passed one has
				//an unrestricted value of true.
				return this.Copy();
			}
				//Catch an InvalidCastException.
				//Throw ArgumentException to let user know.
			catch (InvalidCastException)
			{
				throw new ArgumentException("Argument_WrongType", this.GetType().FullName);
			}                
		}

		public override bool IsSubsetOf(IPermission target)
		{
			//If nothing was passed and unrestricted is false,
			//then return true. 
			if(null == target)
			{
				return !this.unrestricted;
			}
			try
			{        
				//Create a new instance of CustomPermission from the passed object.
				CustomPermission passedpermission = (CustomPermission)target;

				//If unrestricted has the same value in both objects, then
				//one is the subset of the other.
				if(this.unrestricted == passedpermission.unrestricted)
				{
					return true;
				}
				else
				{
					return false;
				} 
			}
				//Catch an InvalidCastException.
				//Throw ArgumentException to let user know.
			catch (InvalidCastException)
			{
				throw new ArgumentException("Argument_WrongType", this.GetType().FullName);
			}                    
		}

		public override void FromXml(SecurityElement PassedElement)
		{
			//Get the unrestricted value from the XML and initialize 
			//the current instance of unrestricted to that value.
			string element = PassedElement.Attribute("Unrestricted");         
 
			if(null != element)
			{  
				this.unrestricted = Convert.ToBoolean(element);
			}
		}

		public override SecurityElement ToXml()
		{
			//Encode the current permission to XML using the 
			//security element class.
			SecurityElement element = new SecurityElement("IPermission");
			Type type = this.GetType();
			StringBuilder AssemblyName = new StringBuilder(type.Assembly.ToString());
			AssemblyName.Replace('\"', '\'');
			element.AddAttribute("class", type.FullName + ", " + AssemblyName);
			element.AddAttribute("version", "1");
			element.AddAttribute("Unrestricted", unrestricted.ToString());
			return element;
		}
	}
	[AttributeUsageAttribute(AttributeTargets.All, AllowMultiple = true)]
	public class CustomPermissionAttribute: CodeAccessSecurityAttribute
	{
		bool unrestricted = false;

		public new bool Unrestricted
		{
			get{ return unrestricted; }
			set{ unrestricted = value; }
		}

		public CustomPermissionAttribute(SecurityAction action): base (action)
		{  
		}
		public override IPermission CreatePermission()
		{
			if(Unrestricted)
			{
				return new CustomPermission(PermissionState.Unrestricted);
			}
			else
			{
				return new CustomPermission(PermissionState.None);
			}
		}
	}
}
